'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let generalizeEQ = (() => {
  var _ref2 = _asyncToGenerator(function* (operation, scope) {
    const lhsIsJoin = isJoinAccessor(operation.lhs);
    const rhsIsJoin = isJoinAccessor(operation.rhs);
    if (!lhsIsJoin && !rhsIsJoin) {
      (0, _debug2.default)("generalizeJoinFilter() (is not a join)");
      // Not part of the join
      return operation;
    }

    if (lhsIsJoin && rhsIsJoin) {
      throw new Error(`In a join, only the lhs or rhs can reference parent`);
    }
    let join;
    let constant;
    if (lhsIsJoin) {
      join = operation.lhs;
      constant = operation.rhs;
    } else {
      join = operation.rhs;
      constant = operation.lhs;
    }
    const joinScopes = yield scope.child({
      value: {}
    }).resolveAccessorForAll(join.path);

    if (joinScopes.length == 0) {
      return {
        op: 'literal',
        value: false
      };
    }
    if (joinScopes.length == 1) {
      return {
        op: 'eq',
        lhs: constant,
        rhs: {
          op: 'literal',
          value: joinScopes[0].value
        }
      };
    }
    return {
      op: 'in',
      lhs: constant,
      rhs: {
        op: 'literal',
        value: joinScopes.map(function (item) {
          return item.value;
        })
      }
    };
  });

  return function generalizeEQ(_x3, _x4) {
    return _ref2.apply(this, arguments);
  };
})();

var _rewrite = require('../plan/rewrite');

var _rewrite2 = _interopRequireDefault(_rewrite);

var _debug = require('../debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

exports.default = (() => {
  var _ref = _asyncToGenerator(function* (node, scope) {
    (0, _debug2.default)('generalizeJoinFilter()', node, scope);
    return (0, _rewrite2.default)(node, function (operation) {
      switch (operation.op) {
        case 'eq':
          return generalizeEQ(operation, scope);
        default:
          return operation;
      }
    });
  });

  function generalizeJoinFilter(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return generalizeJoinFilter;
})();

function isJoinAccessor(operation) {
  if (operation.op != 'accessor') {
    return false;
  }

  if (operation.path.length == 0) {
    return false;
  }

  if (operation.path[0].op == 'parent') {
    return true;
  }

  return false;
}
//# sourceMappingURL=generalizeJoinFilter.js.map